% COMPUTE FIRST-STAGE F-STATISTIC (STOCK & YOGO, 2004)
%
% AGOSTINO (c) Mar 2007 

clear all; clc; close all; pause(.001);
dates   = cal( 1960, 1, 4);
ssmpl   = ical(1960, 1, dates);
esmpl   = ical(2005, 4, dates);

[XLSData, XLSTxT] = xlsread('G:\Research\Favero\WeakInstruments\RawData\CGGEV5.xls', 'cgg2000ev');
K1Q1    = XLSData( ssmpl : esmpl, 2:end);
VarLab  = XLSTxT(  :, 2:end);

TmpData = K1Q1;
TmpText = VarLab;
for i = 1 : size(VarLab, 2)
    disp( [TmpText{1, i}, ' = TmpData(:,', num2str(i), ');'] );
    eval( [TmpText{1, i}, ' = TmpData(:,', num2str(i), ');'] );
end
Dates       = ( 1960.00 : .25 : 2005.75 )';

% INSTRUMENTAL VARIABLES
	p       = 4;
	Z       = [ INF, X, SPREAD, PCM, DM2];
	[ZZ   ] = LagByLags( Z  , p);
    [FF   ] = LagByLags( FFR, p);
    WW      = [ ones( size(ZZ,1), 1), FF(:, 1) ];
    ZZ      = [ FF(:, 2 : p), ZZ ];
    Dates   = Dates(1 + p : end, 1);
    
    
% ENDOGENOUS REGRESSORS
    XXF     = [INF1 XX1 ];
    XXF     = [XXF(p+1 : end, :) ];         % PI(t+k) X(t+q)
    
    XXY     = [INF1 FFR ];
    XXY     = [XXY(p+1 : end, :) ];         % PI(t+k) FFR(t)
    
    XXP     = [XX1 FFR ];
    XXP     = [XXP(p+1 : end, :) ];         %  X(t+q) FFR(t)
    n       = size(XXP, 2);
    
    %! Remark: [XXF XXY ZZ WW ] is consistent: start at 1960:1 + "p" and end at 2006:3
    
% FIRST-STAGE Regression & TESTING WEAK INSTRUMENTS
	PDat    = cal( 1961, 1, 4);             %     Change with p
	sempl   = ical(1979, 3, PDat);          % * % Change with estimation period and p
	eempl   = ical(2000, 4, PDat);          % * %
    Dates   = ( 1961.0 : .25 : 2000.75)';  % * %
    Dates   = Dates( sempl: eempl, :);
    disp(' '); disp([ 'Sample : ', num2str( [ Dates(1),  Dates(end) ])] )
    
    %! DEP. VAR: POLICY RATE
        XFm     = ReX( XXF( sempl:eempl, [1 2] ), WW( sempl:eempl, : ),  1 );
        ZFm     = ReX( ZZ(  sempl:eempl, : )    , WW( sempl:eempl, : ),  1 );
        [T,k]   = size(ZFm);
        LSF     = MReg( XFm, ZFm );
        PZF     = ZFm * ( (ZFm'*ZFm)\eye(k) ) * ZFm';
        VVF     = LSF.MSRE;                           %! <==> Xm'* MZF *Xm / (T-k) given MZF = eye(T) - PZF
        GTF     = chol(VVF) * (XFm'* PZF *XFm / k ) *chol(VVF)';
        disp(' minEval GTF: '); minEval = eig( GTF )'
        
    %! DEP. VAR: OUTPUT GAP
        XYm     = ReX( XXY( sempl:eempl, [1 2] ), WW( sempl:eempl, : ),  1 );
        ZYm     = ReX( ZZ(  sempl:eempl, : )    , WW( sempl:eempl, : ),  1 );
        [T,k]   = size(ZYm);
        LSY     = MReg( XYm, ZYm );
        PZY     = ZYm * ( (ZYm'*ZYm)\eye(k) ) * ZYm';
        VVY     = LSY.MSRE;
        GTY     = chol(VVY) * (XYm'* PZY *XYm / k ) *chol(VVY)';
        disp(' minEval GTY: '); minEval = eig( GTY )'

    %! DEP. VAR: INFLATION RATE
        XPm     = ReX( XXP( sempl:eempl, [1 2] ), WW( sempl:eempl, : ),  1 );
        ZPm     = ReX( ZZ(  sempl:eempl, : )    , WW( sempl:eempl, : ),  1 );
        [T,k]   = size(ZPm);
        LSP     = MReg( XPm, ZPm );
        PZP     = ZPm * ( (ZPm'*ZPm)\eye(k) ) * ZPm';
        VVP     = LSP.MSRE;
        GTP     = chol(VVP) * (XPm'* PZP *XPm / k ) *chol(VVP)';
        disp(' minEval GTP: '); minEval = eig( GTP )'
        
        MatMinEval  = [ eig( GTF )'; eig( GTY )'; eig( GTP )'];
        open MatMinEval
        